import json

from .databaseController import DatabaseController
from .endpoints.query import Archive_Query
from .endpoints.get_post import Archive_GetPost
from .endpoints.get_posts import Archive_GetPosts
from .endpoints.set_action import Archive_SetAction

class Database:
    db : DatabaseController = None

    def __init__(self, api) -> None:
        app = api.app
        if "Archive" in app.databases:
            del app.databases["Archive"]

        self.reload_data()

        app.databases["Archive"] = self

        api.add_resource(Archive_Query, "/Archive/Query")
        api.add_resource(Archive_GetPost, "/Archive/GetPost/<id>")
        api.add_resource(Archive_GetPosts, "/Archive/GetPosts")
        api.add_resource(Archive_SetAction, "/Archive/SetAction")
  
    def get_post(self, id):
        query = f'''SELECT x_posts.id, x_posts.error_id, x_posts.action_taken, x_post_details.text, x_post_details.files, x_post_details.date, accounts.x_handle, accounts.rating from x_posts
        LEFT JOIN x_post_details
            ON x_posts.id = x_post_details.id
        LEFT JOIN accounts
            ON x_posts.account_id = accounts.id WHERE x_posts.id = {id}'''
        result = self.db.run_query(query)
        if len(result) == 0:
            return None
        else:
            #return most recent post
            result = result[-1]
            return {"id": result[0], "id_str": str(result[0]), "error_id": result[1], "action_taken": result[2], "text": result[3], "files": result[4].split(','), "date": result[5], "handle": result[6], "rating": result[7]}

    def get_posts(self, num_posts, actions_taken = [0, 1, 2, 3], last_id = -1, offset = 0):
        num_posts = max(1, min(num_posts, 100))

        where_query = "WHERE x_post_details.id NOT NULL AND x_posts.error_id = 0"
        if last_id != -1:
            where_query += f" AND x_posts.id < {last_id}"
        if actions_taken != [0, 1, 2, 3]:
            where_query += " AND (" + " OR ".join([f"x_posts.action_taken = {action}" for action in actions_taken]) + ")"

        query = f'''
        SELECT x_posts.id, x_posts.action_taken, x_post_details.text, x_post_details.files, x_post_details.date, accounts.x_handle, accounts.rating FROM x_posts 
        LEFT JOIN x_post_details
            ON x_posts.id = x_post_details.id
        LEFT JOIN accounts
            ON x_posts.account_id = accounts.id
        {where_query}
        ORDER BY x_posts.id DESC
        LIMIT {num_posts} OFFSET {offset}'''

        result = self.db.run_query(query)
        posts = []
        if result is not None:
            for result in self.db.run_query(query):
                posts.append({"id": result[0], "id_str": str(result[0]), "action_taken": result[1], "text": result[2], "files": result[3].split(','), "date": result[4], "handle": result[5], "rating": result[6]})
        return posts

    def reload_data(self):
        self.db = DatabaseController("/home/pi/python/Katbots/JapariArchive/database.db")